/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core;

import java.io.IOException;
import java.security.Permission;
import java.security.PrivilegedExceptionAction;
import java.util.logging.Logger;

import javax.security.auth.Subject;

import net.sf.jguard.core.authentication.AccessContext;
import net.sf.jguard.core.authentication.AuthenticationBindings;
import net.sf.jguard.core.authorization.PolicyDecisionPoint;

/**
 * Policy Enforcement Point.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 *
 */
public class PolicyEnforcementPoint {
	
	private AuthenticationBindings authenticationBindings = null;
	private PolicyDecisionPoint policyDecisionPoint = null;
	private static final Logger logger = Logger.getLogger(PolicyEnforcementPoint.class.getName());
	private Permission logonProcess = null;
	private Permission logoff = null;
	private Permission registerProcess = null;
	
	public PolicyEnforcementPoint(AuthenticationBindings authenticationBindings,PolicyDecisionPoint policyDecisionPoint){
		this.authenticationBindings = authenticationBindings;
		this.policyDecisionPoint = policyDecisionPoint;
        logonProcess = authenticationBindings.getLogonProcessPermission();
        logoff = authenticationBindings.getLogoffPermission();
        registerProcess = authenticationBindings.getRegisterProcessPermission();
	}
	
	public void process(AccessContext contxt){
		
		
		  Permission permission = authenticationBindings.getPermissionRequested(contxt);
	        Subject subject = authenticationBindings.getSubject(contxt);
	     
	        if(subject == null){
	        	logger.finest("LAST_ACCESS_DENIED_PERMISSION="+permission);
	        	authenticationBindings.setLastAccessDeniedPermission(contxt,permission);
	        	logger.finest( " subject is null  logonProcess phase ");
	        	authenticationBindings.logonProcess(contxt);
	        }else if(logonProcess.implies(permission)){
	        	logger.finest(" logonProcess phase ");
	        	authenticationBindings.logonProcess(contxt);
	        }else if(logoff.implies(permission)){
	        	logger.finest(" logoff phase ");
	                authenticationBindings.logoff(contxt);
	                //authenticated user(subject!=null) is always authorize to logoff
	        }else if(registerProcess!= null && registerProcess.implies(permission)){
	        	logger.finest(" registerProcess phase ");

	        	//before registration, we authenticate the user as a guest
	        	//=> it "hooks" loginmodules based on credentials because the user is not yer registered
	        	//but permits to execute others loginmodules like captcha one and others to avoid
	        	//annoyances with robot-based mass registration
	        	AccessContext anonymizedContext = authenticationBindings.anonymize(contxt);
	        	boolean authenticate = authenticationBindings.authenticate(anonymizedContext);
	        	if(authenticate== false){
	        		//we don't register the user because it fails loginModules not based on credentials
	        		return;
	        	}
	            authenticationBindings.registerProcess(contxt);
	            //we regrab the Subject because registration has created a new one
	            subject = authenticationBindings.getSubject(contxt);
	        }
	     
				//'decorate' context with authentication informations if needed
				authenticationBindings.process(contxt);
		
				final AccessContext context = contxt;
				final Subject s = subject;
				
				//propagate security information into the Thread
				try {
						Subject.doAsPrivileged(s, new PrivilegedExceptionAction() {
						public Object run() throws IOException{
							//we wrap the ServletRequest to 'correct' the j2ee's JAAS handling
							// according to the j2se way
							try{
								policyDecisionPoint.process(context);
							}catch(Throwable t){
									logger.severe(t.getMessage());
									throw new RuntimeException(t.getMessage());
							}
						  // the 'null' tells the SecurityManager to consider this resource access
						  //in an isolated context, ignoring the permissions of code currently
						  //on the execution stack.
						  return null;
						}
				  },null);
				  } catch (Throwable t) {
					  logger.severe(t.getMessage());
				  }
		
	        

	}
}
